from mlconjug3 import Conjugator
import os
import toml

from .entities import GrammarDescription, VocabularyPlaceholders, VOWELS
from .inflections import verbs_singular_3rd
from .registry import GrammarRegistry
from .vocabulary import Vocabulary


def register_miniphrase():
    cur_dir = os.path.split(__file__)[0]
    miniphrase_dir = os.path.join(cur_dir, "miniphrase")
    register_grammar_topics(miniphrase_dir)


def register_grammar_topics(grammar_dir: str | None = None):
    """
    Read and register descriptions of grammar exercises.
    """

    cur_dir = os.path.split(__file__)[0]
    default_grammar_dir = os.path.join(cur_dir, "topics")
    grammar_dir = grammar_dir or default_grammar_dir

    # iterate over files in `grammar_dir`, load descriptions of topics and exercises and register them.
    for file in os.listdir(grammar_dir):
        if not file.endswith(".toml"):
            continue
        with open(os.path.join(grammar_dir, file), "r") as f:
            grammar_descr_dict = toml.loads(f.read())
            grammar_descr = GrammarDescription(
                name=grammar_descr_dict["name"],
                links=grammar_descr_dict["links"],
                exercises=grammar_descr_dict["exercises"],
            )
            GrammarRegistry.register_grammar_topic(grammar_descr)


def apply_vocabulary(s: str, vocab: Vocabulary) -> str:
    """
    Takes input string and replaces placeholders with respective minilex entities.
    """

    while VocabularyPlaceholders.Verb in s:
        s = s.replace(VocabularyPlaceholders.Verb, vocab.random_verb(), 1)

    while VocabularyPlaceholders.Noun in s:
        s = s.replace(VocabularyPlaceholders.Noun, vocab.random_noun(), 1)

    while VocabularyPlaceholders.NounPlural in s:
        s = s.replace(VocabularyPlaceholders.NounPlural, vocab.random_noun_plural(), 1)

    while VocabularyPlaceholders.NounNonPerson in s:
        s = s.replace(VocabularyPlaceholders.NounNonPerson, vocab.random_noun_non_person(), 1)

    while VocabularyPlaceholders.Number100 in s:
        s = s.replace(VocabularyPlaceholders.Number100, vocab.random_number(max=100), 1)

    while VocabularyPlaceholders.Number1000 in s:
        s = s.replace(VocabularyPlaceholders.Number1000, vocab.random_number(max=1000), 1)

    while VocabularyPlaceholders.Number100k in s:
        s = s.replace(VocabularyPlaceholders.Number100k, vocab.random_number(max=100_000), 1)

    while VocabularyPlaceholders.ANoun in s:
        noun = vocab.random_noun()
        suffix = "an" if noun[0] in VOWELS else "a"
        s = s.replace(VocabularyPlaceholders.ANoun, f"{suffix} {noun}", 1)

    while VocabularyPlaceholders.ANounNonPerson in s:
        noun = vocab.random_noun_non_person()
        suffix = "an" if noun[0] in VOWELS else "a"
        s = s.replace(VocabularyPlaceholders.ANounNonPerson, f"{suffix} {noun}", 1)

    while VocabularyPlaceholders.Weekday in s:
        s = s.replace(VocabularyPlaceholders.Weekday, vocab.random_weekday(), 1)

    while VocabularyPlaceholders.Season in s:
        s = s.replace(VocabularyPlaceholders.Season, vocab.random_season(), 1)

    while VocabularyPlaceholders.Month in s:
        s = s.replace(VocabularyPlaceholders.Month, vocab.random_month(), 1)

    while VocabularyPlaceholders.Adjective in s:
        s = s.replace(VocabularyPlaceholders.Adjective, vocab.random_adjective(), 1)

    while VocabularyPlaceholders.Adverb in s:
        s = s.replace(VocabularyPlaceholders.Adverb, vocab.random_adverb(), 1)

    while VocabularyPlaceholders.Country in s:
        s = s.replace(VocabularyPlaceholders.Country, vocab.random_country(), 1)

    while VocabularyPlaceholders.Place in s:
        s = s.replace(VocabularyPlaceholders.Place, vocab.random_place(), 1)

    while VocabularyPlaceholders.City in s:
        s = s.replace(VocabularyPlaceholders.City, vocab.random_city(), 1)

    while VocabularyPlaceholders.Nationality in s:
        s = s.replace(VocabularyPlaceholders.Nationality, vocab.random_nationality(), 1)

    conj = Conjugator(language="en")
    while VocabularyPlaceholders.VerbSingular3rd in s:
        cur_verb = conj.conjugate(vocab.random_verb())
        conj_verb = verbs_singular_3rd.get(
            cur_verb.name,
            cur_verb["indicative"]["indicative present"]["he/she/it"]
        )
        s = s.replace(VocabularyPlaceholders.VerbSingular3rd, conj_verb, 1)

    while VocabularyPlaceholders.VerbPast in s:
        cur_verb = conj.conjugate(vocab.random_verb())
        conj_verb = cur_verb["indicative"]["indicative past tense"]["I"]
        s = s.replace(VocabularyPlaceholders.VerbPast, conj_verb, 1)

    while VocabularyPlaceholders.VerbProgressive in s:
        cur_verb = conj.conjugate(vocab.random_verb())
        conj_verb = cur_verb["indicative"]["indicative present continuous"]["I"]
        s = s.replace(VocabularyPlaceholders.VerbProgressive, conj_verb, 1)

    return s



"""
Factory for generation of exercises for the given topic.
"""

from __future__ import annotations

from .entities import GrammarDescription


class GrammarRegistry:
    topics: dict[str, GrammarDescription] = dict()

    @staticmethod
    def register_grammar_topic(grammar_descr: GrammarDescription):
        if grammar_descr.name in GrammarRegistry.topics:
            return
        GrammarRegistry.topics[grammar_descr.name] = grammar_descr

    @staticmethod
    def get_registered_topics() -> list[str]:
        """
        Returns names of registered topics.
        """
        return list(GrammarRegistry.topics)

    @staticmethod
    def generate_answers(exercises: list[str], grammar_topic: str | None = None) -> list[list[str]]:
        """
        Generate list of possible answers for the given exercises.
        Each exercise can have several possible answers.
        """
        ...



"""
Class for managing interchangeable vocabulary.
"""

from __future__ import annotations

import inflect
import os
import random
import toml


p = inflect.engine()


class Vocabulary:
    vocab_file: str  # file with the current vocabulary
    adjectives: list[str]
    adverbs: list[str]
    nouns: list[str]
    verbs: list[str]
    weekdays: list[str] = [
        "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"
    ]
    seasons = [
        "Spring", "Summer", "Autumn", "Winter"
    ]
    months = [
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December"
    ]
    cities = [
        "Beijing", "Berlin", "Busan", "Daejeon", "London", "New York", "Paris", "Seoul", "Tokyo"
    ]
    countries = [
        "Australia", "Brazil", "Canada", "China", "England", "France", "Germany", "India", "Japan",
        "Korea", "Philippines", "Russia", "Thailand", "Vietnam", "USA"
    ]
    nationalities = [
        "Australian", "Brazilian", "Canadian", "Chinese", "English", "French", "German", "Indian", "Japanese",
        "Korean", "Filipino", "Russian", "Thai", "Vietnamese", "American"
    ]
    person_nouns = [
        "anybody", "boy", "brother", "child", "daughter", "everybody",
        "family", "father", "friend", "girl", "husband", "lady", "man", "mother",
        "nobody", "people", "person", "relative", "sister", "somebody", "son",
        "teacher", "wife", "woman"
    ]
    places = [
        "airport", "apartment", "country", "flat", "home", "house", "road", "room",
        "school", "shop", "side", "street", "town", "work", "bathroom"
    ]

    def __init__(self, vocab_file: str):
        self.vocab_file = vocab_file
        self._load()

    @staticmethod
    def load(vocab_file: str) -> Vocabulary:
        return Vocabulary(vocab_file)

    def _load(self):
        with open(self.vocab_file, "r") as f:
            vocab_dict = toml.loads(f.read())
            self.adjectives = vocab_dict.get("adjectives") or []
            self.adverbs = vocab_dict.get("adverbs") or []
            self.nouns = vocab_dict.get("nouns") or []
            self.verbs = vocab_dict.get("verbs") or []

    def random_verb(self) -> str:
        return random.choice(self.verbs)

    def random_noun(self) -> str:
        return random.choice(self.nouns)

    def random_noun_plural(self) -> str:
        noun = random.choice(self.nouns)
        return p.plural_noun(noun)

    def random_noun_non_person(self) -> str:
        res = random.choice(self.nouns)
        while res in Vocabulary.person_nouns:
            res = random.choice(self.nouns)
        return res

    def random_adjective(self) -> str:
        return random.choice(self.adjectives)

    def random_adverb(self) -> str:
        return random.choice(self.adverbs)

    def random_weekday(self) -> str:
        return random.choice(self.weekdays)

    def random_season(self) -> str:
        return random.choice(self.seasons)

    def random_month(self) -> str:
        return random.choice(self.months)

    def random_number(self, max) -> str:
        return str(random.randint(0, max))

    def random_place(self) -> str:
        return random.choice(self.places)

    def random_city(self) -> str:
        return random.choice(self.cities)

    def random_country(self) -> str:
        return random.choice(self.countries)

    def random_nationality(self) -> str:
        return random.choice(self.nationalities)


def load_minilex() -> Vocabulary:
    cur_dir = os.path.split(__file__)[0]
    return Vocabulary.load(os.path.join(cur_dir, "vocabs", "minilex.toml"))
